import geojson
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from datapoint.exceptions import APIException
from datapoint.Forecast import Forecast

API_URL = "https://data.hub.api.metoffice.gov.uk/sitespecific/v0/point/"


class Manager:
    """Manager for DataHub connection.

    Wraps calls to DataHub API, and provides Forecast objects. Basic Usage:

    ::

      >>> import datapoint
      >>> m = datapoint.Manager.Manager(api_key = "blah")
      >>> f = m.get_forecast(
                  latitude=50,
                  longitude=0,
                  frequency="hourly",
                  convert_weather_code=True
              )
      >>> f.now()
      {
          'time': datetime.datetime(2024, 2, 19, 13, 0, tzinfo=datetime.timezone.utc),
          'screenTemperature': {
              'value': 10.09,
              'description': 'Screen Air Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'screenDewPointTemperature': {
              'value': 8.08,
              'description': 'Screen Dew Point Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'feelsLikeTemperature': {
              'value': 6.85,
              'description': 'Feels Like Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'windSpeed10m': {
              'value': 7.57,
              'description': '10m Wind Speed',
              'unit_name': 'metres per second',
              'unit_symbol': 'm/s'
          },
          'windDirectionFrom10m': {
              'value': 263,
              'description': '10m Wind From Direction',
              'unit_name': 'degrees',
              'unit_symbol': 'deg'
          },
          'windGustSpeed10m': {
              'value': 12.31,
              'description': '10m Wind Gust Speed',
              'unit_name': 'metres per second',
              'unit_symbol': 'm/s'
          },
          'visibility': {
              'value': 21201,
              'description': 'Visibility',
              'unit_name': 'metres',
              'unit_symbol': 'm'
          },
          'screenRelativeHumidity': {
              'value': 87.81,
              'description': 'Screen Relative Humidity',
              'unit_name': 'percentage',
              'unit_symbol': '%'
          },
          'mslp': {
              'value': 103080,
              'description': 'Mean Sea Level Pressure',
              'unit_name': 'pascals',
              'unit_symbol': 'Pa'
          },
          'uvIndex': {
              'value': 1,
              'description': 'UV Index',
              'unit_name': 'dimensionless',
              'unit_symbol': '1'
          },
          'significantWeatherCode': {
              'value': 'Cloudy',
              'description': 'Significant Weather Code',
              'unit_name': 'dimensionless',
              'unit_symbol': '1'
          },
          'precipitationRate': {
              'value': 0.0,
              'description': 'Precipitation Rate',
              'unit_name': 'millimetres per hour',
              'unit_symbol': 'mm/h'
          },
          'probOfPrecipitation': {
              'value': 21,
              'description': 'Probability of Precipitation',
              'unit_name': 'percentage',
              'unit_symbol': '%'
          }
      }

    """

    def __init__(self, api_key=""):
        self.api_key = api_key

    def __get_retry_session(
        self,
        retries=10,
        backoff_factor=0.3,
        status_forcelist=(500, 502, 504),
        session=None,
    ):
        """
        Retry the connection using requests if it fails. Use this as a wrapper
        to request from datapoint. See
        https://requests.readthedocs.io/en/latest/user/advanced/?highlight=retry#example-automatic-retries
        for more details.

        :parameter retries: How many times to retry
        :parameter backoff_factor: Backoff between attempts after second try
        :parameter status_forcelist: Codes to force a retry on
        :parameter session: Existing session to use

        :return: Session object
        :rtype: <class 'requests.sessions.Session'>
        """

        # requests.Session allows finer control, which is needed to use the
        # retrying code
        the_session = session or requests.Session()

        # The Retry object manages the actual retrying
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
        )

        adapter = HTTPAdapter(max_retries=retry)

        the_session.mount("http://", adapter)
        the_session.mount("https://", adapter)

        return the_session

    def __call_api(self, latitude, longitude, frequency):
        """
        Call the datapoint api using the requests module

        :parameter latitude: Latitude of forecast location
        :parameter longitude: Longitude of forecast location
        :parameter frequency: Forecast frequency. One of 'hourly', 'three-hourly, 'daily'
        :type latitude: float
        :type longitude: float
        :type frequency: string

        :return: Data from DataPoint
        :rtype: dict
        """
        params = {
            "latitude": latitude,
            "longitude": longitude,
            "includeLocationName": True,
            "excludeParameterMetadata": False,
        }
        headers = {
            "accept": "application/json",
            "apikey": self.api_key,
        }

        if frequency == "twice-daily":
            request_url = API_URL + "daily"
        else:
            request_url = API_URL + frequency

        # Add a timeout to the request.
        # The value of 1 second is based on attempting 100 connections to
        # datapoint and taking ten times the mean connection time (rounded up).
        # Could expose to users in the functions which need to call the api.
        # req = requests.get(url, params=payload, timeout=1)
        # The wrapper function __retry_session returns a requests.Session
        # object. This has a .get() function like requests.get(), so the use
        # doesn't change here.

        sess = self.__get_retry_session()
        req = sess.get(
            request_url,
            params=params,
            headers=headers,
            timeout=1,
        )

        req.raise_for_status()

        try:
            data = geojson.loads(req.text)
        except ValueError as exc:
            raise APIException("DataPoint has not returned valid JSON") from exc

        return data

    def get_forecast(
        self, latitude, longitude, frequency="daily", convert_weather_code=True
    ):
        """
        Get a forecast for the provided site. Three frequencies are supported
        by DataHub: hourly, three-hourly and daily. The 'twice-daily' option is
        for convenience and splits a daily forecast into two steps, one for day
        and one for night.

        :parameter latitude: Latitude of forecast location
        :parameter longitude: Longitude of forecast location
        :parameter frequency: Forecast frequency. One of 'hourly',
            'three-hourly,'twice-daily', 'daily'
        :parameter convert_weather_code: Convert numeric weather codes to string description
        :type latitude: float
        :type longitude: float
        :type frequency: string
        :type convert_weather_code: bool

        :return: :class: `Forecast <Forecast>` object
        :rtype: datapoint.Forecast
        """
        if frequency not in ["hourly", "three-hourly", "twice-daily", "daily"]:
            raise ValueError(
                "frequency must be set to one of 'hourly', 'three-hourly', "
                "'twice-daily', 'daily'"
            )
        data = self.__call_api(latitude, longitude, frequency)
        forecast = Forecast(
            frequency=frequency,
            api_data=data,
            convert_weather_code=convert_weather_code,
        )

        return forecast



# See https://datahub.metoffice.gov.uk/support/faqs for definitions
WEATHER_CODES = {
    "NA": "Not available",
    "0": "Clear night",
    "1": "Sunny day",
    "2": "Partly cloudy",
    "3": "Partly cloudy",
    "4": "Not used",
    "5": "Mist",
    "6": "Fog",
    "7": "Cloudy",
    "8": "Overcast",
    "9": "Light rain shower",
    "10": "Light rain shower",
    "11": "Drizzle",
    "12": "Light rain",
    "13": "Heavy rain shower",
    "14": "Heavy rain shower",
    "15": "Heavy rain",
    "16": "Sleet shower",
    "17": "Sleet shower",
    "18": "Sleet",
    "19": "Hail shower",
    "20": "Hail shower",
    "21": "Hail",
    "22": "Light snow shower",
    "23": "Light snow shower",
    "24": "Light snow",
    "25": "Heavy snow shower",
    "26": "Heavy snow shower",
    "27": "Heavy snow",
    "28": "Thunder shower",
    "29": "Thunder shower",
    "30": "Thunder",
}



#!/usr/bin/env python
"""
This example will print out a simple forecast for the next 5 days.
It will allow us to explore the day, timestep and element objects.
"""

import datetime

import datapoint

# Create datapoint connection
manager = datapoint.Manager(api_key="api key goes here")


forecast = manager.get_forecast(51.500728, -0.124626, frequency="hourly")

# Loop through timesteps and print information
for timestep in forecast.timesteps:
    print(timestep["time"])
    print(timestep["significantWeatherCode"]["value"])
    print(
        "{temp} {temp_units}".format(
            temp=timestep["screenTemperature"]["value"],
            temp_units=timestep["screenTemperature"]["unit_symbol"],
        )
    )

print(forecast.now())

print(forecast.at_datetime(datetime.datetime(2024, 2, 11, 14, 0)))



import datetime

EXPECTED_FIRST_HOURLY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 15, 19, 0, tzinfo=datetime.timezone.utc),
    "screenTemperature": {
        "value": 11.0,
        "description": "Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "maxScreenAirTemp": {
        "value": 11.55,
        "description": "Maximum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 10.98,
        "description": "Minimum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "screenDewPointTemperature": {
        "value": 8.94,
        "description": "Screen Dew Point Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "feelsLikeTemperature": {
        "value": 10.87,
        "description": "Feels Like Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "windSpeed10m": {
        "value": 1.18,
        "description": "10m Wind Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 180,
        "description": "10m Wind From Direction",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 6.69,
        "description": "10m Wind Gust Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "max10mWindGust": {
        "value": 8.92,
        "description": "Maximum 10m Wind Gust Speed Over Previous Hour",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 19174,
        "description": "Visibility",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "screenRelativeHumidity": {
        "value": 86.99,
        "description": "Screen Relative Humidity",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "mslp": {
        "value": 100660,
        "description": "Mean Sea Level Pressure",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "uvIndex": {
        "value": 0,
        "description": "UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "significantWeatherCode": {
        "value": "Partly cloudy",
        "description": "Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "precipitationRate": {
        "value": 0.0,
        "description": "Precipitation Rate",
        "unit_name": "millimetres per hour",
        "unit_symbol": "mm/h",
    },
    "totalPrecipAmount": {
        "value": 0.0,
        "description": "Total Precipitation Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "probOfPrecipitation": {
        "value": 4,
        "description": "Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_FIRST_HOURLY_TIMESTEP_RAW_WEATHER_CODE = {
    "time": datetime.datetime(2024, 2, 15, 19, 0, tzinfo=datetime.timezone.utc),
    "screenTemperature": {
        "value": 11.0,
        "description": "Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "maxScreenAirTemp": {
        "value": 11.55,
        "description": "Maximum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 10.98,
        "description": "Minimum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "screenDewPointTemperature": {
        "value": 8.94,
        "description": "Screen Dew Point Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "feelsLikeTemperature": {
        "value": 10.87,
        "description": "Feels Like Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "windSpeed10m": {
        "value": 1.18,
        "description": "10m Wind Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 180,
        "description": "10m Wind From Direction",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 6.69,
        "description": "10m Wind Gust Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "max10mWindGust": {
        "value": 8.92,
        "description": "Maximum 10m Wind Gust Speed Over Previous Hour",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 19174,
        "description": "Visibility",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "screenRelativeHumidity": {
        "value": 86.99,
        "description": "Screen Relative Humidity",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "mslp": {
        "value": 100660,
        "description": "Mean Sea Level Pressure",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "uvIndex": {
        "value": 0,
        "description": "UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "significantWeatherCode": {
        "value": 2,
        "description": "Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "precipitationRate": {
        "value": 0.0,
        "description": "Precipitation Rate",
        "unit_name": "millimetres per hour",
        "unit_symbol": "mm/h",
    },
    "totalPrecipAmount": {
        "value": 0.0,
        "description": "Total Precipitation Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "probOfPrecipitation": {
        "value": 4,
        "description": "Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_HOURLY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 16, 19, 0, tzinfo=datetime.timezone.utc),
    "screenTemperature": {
        "value": 8.94,
        "description": "Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "maxScreenAirTemp": {
        "value": 9.6,
        "description": "Maximum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 8.9,
        "description": "Minimum Screen Air Temperature Over Previous Hour",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "screenDewPointTemperature": {
        "value": 6.8,
        "description": "Screen Dew Point Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "feelsLikeTemperature": {
        "value": 7.68,
        "description": "Feels Like Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "windSpeed10m": {
        "value": 2.46,
        "description": "10m Wind Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 255,
        "description": "10m Wind From Direction",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 7.35,
        "description": "10m Wind Gust Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "max10mWindGust": {
        "value": 8.02,
        "description": "Maximum 10m Wind Gust Speed Over Previous Hour",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 22767,
        "description": "Visibility",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "screenRelativeHumidity": {
        "value": 86.81,
        "description": "Screen Relative Humidity",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "mslp": {
        "value": 102246,
        "description": "Mean Sea Level Pressure",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "uvIndex": {
        "value": 0,
        "description": "UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "significantWeatherCode": {
        "value": "Clear night",
        "description": "Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "precipitationRate": {
        "value": 0.0,
        "description": "Precipitation Rate",
        "unit_name": "millimetres per hour",
        "unit_symbol": "mm/h",
    },
    "totalPrecipAmount": {
        "value": 0.0,
        "description": "Total Precipitation Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Hour",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "probOfPrecipitation": {
        "value": 0,
        "description": "Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_HOURLY_FINAL_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 17, 19, 0, tzinfo=datetime.timezone.utc),
    "screenTemperature": {
        "value": 10.34,
        "description": "Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "screenDewPointTemperature": {
        "value": 9.37,
        "description": "Screen Dew Point Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "feelsLikeTemperature": {
        "value": 8.34,
        "description": "Feels Like Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "windSpeed10m": {
        "value": 4.29,
        "description": "10m Wind Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 187,
        "description": "10m Wind From Direction",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 8.68,
        "description": "10m Wind Gust Speed",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 10128,
        "description": "Visibility",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "screenRelativeHumidity": {
        "value": 94.17,
        "description": "Screen Relative Humidity",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "mslp": {
        "value": 102910,
        "description": "Mean Sea Level Pressure",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "uvIndex": {
        "value": 0,
        "description": "UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "significantWeatherCode": {
        "value": "Overcast",
        "description": "Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "precipitationRate": {
        "value": 0.0,
        "description": "Precipitation Rate",
        "unit_name": "millimetres per hour",
        "unit_symbol": "mm/h",
    },
    "probOfPrecipitation": {
        "value": 16,
        "description": "Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}
EXPECTED_FIRST_DAILY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 16, 0, 0, tzinfo=datetime.timezone.utc),
    "midday10MWindSpeed": {
        "value": 5.04,
        "description": "10m Wind Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindSpeed": {
        "value": 1.39,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midday10MWindDirection": {
        "value": 273,
        "description": "10m Wind Direction at Local Midday",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindDirection": {
        "value": 243,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midday10MWindGust": {
        "value": 8.75,
        "description": "10m Wind Gust Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindGust": {
        "value": 7.2,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "middayVisibility": {
        "value": 28772,
        "description": "Visibility at Local Midday",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightVisibility": {
        "value": 27712,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "middayRelativeHumidity": {
        "value": 75.21,
        "description": "Relative Humidity at Local Midday",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightRelativeHumidity": {
        "value": 80.91,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "middayMslp": {
        "value": 101680,
        "description": "Mean Sea Level Pressure at Local Midday",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "midnightMslp": {
        "value": 102640,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "nightSignificantWeatherCode": {
        "value": "Cloudy",
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "dayMaxScreenTemperature": {
        "value": 12.82,
        "description": "Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinScreenTemperature": {
        "value": 5.32,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxTemp": {
        "value": 14.1,
        "description": "Upper Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 9.17,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxTemp": {
        "value": 11.97,
        "description": "Lower Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 3.56,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 6.27,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxFeelsLikeTemp": {
        "value": 12.47,
        "description": "Upper Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.74,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxFeelsLikeTemp": {
        "value": 10.01,
        "description": "Lower Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 2.75,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 11,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 10,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 0,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_FIRST_DAILY_TIMESTEP_RAW_WEATHER_CODE = {
    "time": datetime.datetime(2024, 2, 16, 0, 0, tzinfo=datetime.timezone.utc),
    "midday10MWindSpeed": {
        "value": 5.04,
        "description": "10m Wind Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindSpeed": {
        "value": 1.39,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midday10MWindDirection": {
        "value": 273,
        "description": "10m Wind Direction at Local Midday",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindDirection": {
        "value": 243,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midday10MWindGust": {
        "value": 8.75,
        "description": "10m Wind Gust Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindGust": {
        "value": 7.2,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "middayVisibility": {
        "value": 28772,
        "description": "Visibility at Local Midday",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightVisibility": {
        "value": 27712,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "middayRelativeHumidity": {
        "value": 75.21,
        "description": "Relative Humidity at Local Midday",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightRelativeHumidity": {
        "value": 80.91,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "middayMslp": {
        "value": 101680,
        "description": "Mean Sea Level Pressure at Local Midday",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "midnightMslp": {
        "value": 102640,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "nightSignificantWeatherCode": {
        "value": 7,
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "dayMaxScreenTemperature": {
        "value": 12.82,
        "description": "Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinScreenTemperature": {
        "value": 5.32,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxTemp": {
        "value": 14.1,
        "description": "Upper Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 9.17,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxTemp": {
        "value": 11.97,
        "description": "Lower Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 3.56,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 6.27,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxFeelsLikeTemp": {
        "value": 12.47,
        "description": "Upper Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.74,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxFeelsLikeTemp": {
        "value": 10.01,
        "description": "Lower Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 2.75,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 11,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 10,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 0,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_DAILY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 17, 0, 0, tzinfo=datetime.timezone.utc),
    "midday10MWindSpeed": {
        "value": 4.32,
        "description": "10m Wind Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindSpeed": {
        "value": 6.1,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midday10MWindDirection": {
        "value": 230,
        "description": "10m Wind Direction at Local Midday",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindDirection": {
        "value": 218,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midday10MWindGust": {
        "value": 8.75,
        "description": "10m Wind Gust Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindGust": {
        "value": 12.98,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "middayVisibility": {
        "value": 4158,
        "description": "Visibility at Local Midday",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightVisibility": {
        "value": 5915,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "middayRelativeHumidity": {
        "value": 97.38,
        "description": "Relative Humidity at Local Midday",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightRelativeHumidity": {
        "value": 93.62,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "middayMslp": {
        "value": 103140,
        "description": "Mean Sea Level Pressure at Local Midday",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "midnightMslp": {
        "value": 102800,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "maxUvIndex": {
        "value": 1,
        "description": "Day Maximum UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "daySignificantWeatherCode": {
        "value": "Overcast",
        "description": "Day Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "nightSignificantWeatherCode": {
        "value": "Heavy rain",
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "dayMaxScreenTemperature": {
        "value": 12.0,
        "description": "Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinScreenTemperature": {
        "value": 9.96,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxTemp": {
        "value": 13.71,
        "description": "Upper Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 10.71,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxTemp": {
        "value": 10.23,
        "description": "Lower Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 9.04,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayMaxFeelsLikeTemp": {
        "value": 10.6,
        "description": "Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 7.76,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxFeelsLikeTemp": {
        "value": 11.49,
        "description": "Upper Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.28,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxFeelsLikeTemp": {
        "value": 8.38,
        "description": "Lower Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 7.04,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayProbabilityOfPrecipitation": {
        "value": 18,
        "description": "Probability of Precipitation During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 97,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfRain": {
        "value": 18,
        "description": "Probability of Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 97,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavyRain": {
        "value": 5,
        "description": "Probability of Heavy Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 96,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 20,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 10,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_DAILY_FINAL_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 18, 0, 0, tzinfo=datetime.timezone.utc),
    "midday10MWindSpeed": {
        "value": 4.07,
        "description": "10m Wind Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindSpeed": {
        "value": 2.84,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midday10MWindDirection": {
        "value": 292,
        "description": "10m Wind Direction at Local Midday",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindDirection": {
        "value": 281,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midday10MWindGust": {
        "value": 7.55,
        "description": "10m Wind Gust Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindGust": {
        "value": 6.36,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "middayVisibility": {
        "value": 25425,
        "description": "Visibility at Local Midday",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightVisibility": {
        "value": 17183,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "middayRelativeHumidity": {
        "value": 86.19,
        "description": "Relative Humidity at Local Midday",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightRelativeHumidity": {
        "value": 94.68,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "middayMslp": {
        "value": 102625,
        "description": "Mean Sea Level Pressure at Local Midday",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "midnightMslp": {
        "value": 103041,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "maxUvIndex": {
        "value": 1,
        "description": "Day Maximum UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "daySignificantWeatherCode": {
        "value": "Light rain",
        "description": "Day Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "nightSignificantWeatherCode": {
        "value": "Cloudy",
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "dayMaxScreenTemperature": {
        "value": 13.89,
        "description": "Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinScreenTemperature": {
        "value": 7.15,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxTemp": {
        "value": 14.73,
        "description": "Upper Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 9.16,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxTemp": {
        "value": 12.4,
        "description": "Lower Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 5.31,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayMaxFeelsLikeTemp": {
        "value": 11.75,
        "description": "Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 5.34,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxFeelsLikeTemp": {
        "value": 13.47,
        "description": "Upper Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 7.12,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxFeelsLikeTemp": {
        "value": 10.8,
        "description": "Lower Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 5.06,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayProbabilityOfPrecipitation": {
        "value": 55,
        "description": "Probability of Precipitation During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 9,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfRain": {
        "value": 55,
        "description": "Probability of Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 9,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavyRain": {
        "value": 36,
        "description": "Probability of Heavy Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 2,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHail": {
        "value": 4,
        "description": "Probability of Hail During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSferics": {
        "value": 3,
        "description": "Probability of Sferics During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_FIRST_THREE_HOURLY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 17, 15, 0, tzinfo=datetime.timezone.utc),
    "maxScreenAirTemp": {
        "value": 12.0,
        "description": "Maximum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 10.73,
        "description": "Minimum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "max10mWindGust": {
        "value": 8.87,
        "description": "Maximum 10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "significantWeatherCode": {
        "value": "Overcast",
        "description": "Three Hour Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "totalPrecipAmount": {
        "value": 0.0,
        "description": "Total Precipitation Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "windSpeed10m": {
        "value": 5.04,
        "description": "10m Wind Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 225,
        "description": "10m Wind From Direction Over Previous Three Hours",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 8.75,
        "description": "10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 9118,
        "description": "Visibility Over Previous Three Hours",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "mslp": {
        "value": 103090,
        "description": "Mean Sea Level Pressure Over Previous Three Hours",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "screenRelativeHumidity": {
        "value": 96.84,
        "description": "Screen Relative Humidity Over Previous Three Hours",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "feelsLikeTemp": {
        "value": 8.53,
        "description": "Feels Like Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "uvIndex": {
        "value": 1,
        "description": "Maximum UV Index Over Previous Three Hours",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "probOfPrecipitation": {
        "value": 10,
        "description": "Three Hour Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSnow": {
        "value": 0,
        "description": "Three Hour Probability of Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavySnow": {
        "value": 0,
        "description": "Three Hour Probability of Heavy Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfRain": {
        "value": 10,
        "description": "Three Hour Probability of Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavyRain": {
        "value": 0,
        "description": "Three Hour Probability of Heavy Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHail": {
        "value": 0,
        "description": "Three Hour Probability of Hail",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSferics": {
        "value": 0,
        "description": "Three Hour Probability of Sferics",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_THREE_HOURLY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 22, 18, 0, tzinfo=datetime.timezone.utc),
    "maxScreenAirTemp": {
        "value": 9.82,
        "description": "Maximum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 7.61,
        "description": "Minimum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "max10mWindGust": {
        "value": 19.28,
        "description": "Maximum 10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "significantWeatherCode": {
        "value": "Light rain shower",
        "description": "Three Hour Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "totalPrecipAmount": {
        "value": 1.11,
        "description": "Total Precipitation Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "windSpeed10m": {
        "value": 7.91,
        "description": "10m Wind Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 239,
        "description": "10m Wind From Direction Over Previous Three Hours",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 14.42,
        "description": "10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 20148,
        "description": "Visibility Over Previous Three Hours",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "mslp": {
        "value": 98434,
        "description": "Mean Sea Level Pressure Over Previous Three Hours",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "screenRelativeHumidity": {
        "value": 79.36,
        "description": "Screen Relative Humidity Over Previous Three Hours",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "feelsLikeTemp": {
        "value": 3.74,
        "description": "Feels Like Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "uvIndex": {
        "value": 0,
        "description": "Maximum UV Index Over Previous Three Hours",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "probOfPrecipitation": {
        "value": 44,
        "description": "Three Hour Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSnow": {
        "value": 0,
        "description": "Three Hour Probability of Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavySnow": {
        "value": 0,
        "description": "Three Hour Probability of Heavy Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfRain": {
        "value": 44,
        "description": "Three Hour Probability of Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavyRain": {
        "value": 32,
        "description": "Three Hour Probability of Heavy Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHail": {
        "value": 4,
        "description": "Three Hour Probability of Hail",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSferics": {
        "value": 6,
        "description": "Three Hour Probability of Sferics",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_THREE_HOURLY_FINAL_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 24, 15, 0, tzinfo=datetime.timezone.utc),
    "maxScreenAirTemp": {
        "value": 8.25,
        "description": "Maximum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "minScreenAirTemp": {
        "value": 7.96,
        "description": "Minimum Screen Air Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "max10mWindGust": {
        "value": 12.77,
        "description": "Maximum 10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "significantWeatherCode": {
        "value": "Light rain shower",
        "description": "Three Hour Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "totalPrecipAmount": {
        "value": 0.58,
        "description": "Total Precipitation Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "totalSnowAmount": {
        "value": 0,
        "description": "Total Snow Amount Over Previous Three Hours",
        "unit_name": "millimetres",
        "unit_symbol": "mm",
    },
    "windSpeed10m": {
        "value": 5.98,
        "description": "10m Wind Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "windDirectionFrom10m": {
        "value": 228,
        "description": "10m Wind From Direction Over Previous Three Hours",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "windGustSpeed10m": {
        "value": 11.48,
        "description": "10m Wind Gust Speed Over Previous Three Hours",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "visibility": {
        "value": 22483,
        "description": "Visibility Over Previous Three Hours",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "mslp": {
        "value": 99725,
        "description": "Mean Sea Level Pressure Over Previous Three Hours",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "screenRelativeHumidity": {
        "value": 72.01,
        "description": "Screen Relative Humidity Over Previous Three Hours",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "feelsLikeTemp": {
        "value": 4.97,
        "description": "Feels Like Temperature Over Previous Three Hours",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "uvIndex": {
        "value": 1,
        "description": "Maximum UV Index Over Previous Three Hours",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "probOfPrecipitation": {
        "value": 32,
        "description": "Three Hour Probability of Precipitation",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSnow": {
        "value": 0,
        "description": "Three Hour Probability of Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavySnow": {
        "value": 0,
        "description": "Three Hour Probability of Heavy Snow",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfRain": {
        "value": 32,
        "description": "Three Hour Probability of Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHeavyRain": {
        "value": 17,
        "description": "Three Hour Probability of Heavy Rain",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfHail": {
        "value": 1,
        "description": "Three Hour Probability of Hail",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "probOfSferics": {
        "value": 3,
        "description": "Three Hour Probability of Sferics",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_FIRST_TWICE_DAILY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 16, 0, 0, tzinfo=datetime.timezone.utc),
    "midnight10MWindSpeed": {
        "value": 1.39,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindDirection": {
        "value": 243,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindGust": {
        "value": 7.2,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnightVisibility": {
        "value": 27712,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightRelativeHumidity": {
        "value": 80.91,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightMslp": {
        "value": 102640,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "nightSignificantWeatherCode": {
        "value": "Cloudy",
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "nightMinScreenTemperature": {
        "value": 5.32,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 9.17,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 3.56,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 6.27,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.74,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 2.75,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 11,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 10,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 0,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_TWICE_DAILY_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 17, 0, 0, tzinfo=datetime.timezone.utc),
    "midnight10MWindSpeed": {
        "value": 6.1,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindDirection": {
        "value": 218,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindGust": {
        "value": 12.98,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnightVisibility": {
        "value": 5915,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightRelativeHumidity": {
        "value": 93.62,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightMslp": {
        "value": 102800,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "nightSignificantWeatherCode": {
        "value": "Heavy rain",
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "nightMinScreenTemperature": {
        "value": 9.96,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 10.71,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 9.04,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 7.76,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.28,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 7.04,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 97,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 97,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 96,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 20,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 10,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_AT_DATETIME_TWICE_DAILY_FINAL_TIMESTEP = {
    "time": datetime.datetime(2024, 2, 17, 12, 0, tzinfo=datetime.timezone.utc),
    "midday10MWindSpeed": {
        "value": 4.32,
        "description": "10m Wind Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midday10MWindDirection": {
        "value": 230,
        "description": "10m Wind Direction at Local Midday",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midday10MWindGust": {
        "value": 8.75,
        "description": "10m Wind Gust Speed at Local Midday",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "middayVisibility": {
        "value": 4158,
        "description": "Visibility at Local Midday",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "middayRelativeHumidity": {
        "value": 97.38,
        "description": "Relative Humidity at Local Midday",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "middayMslp": {
        "value": 103140,
        "description": "Mean Sea Level Pressure at Local Midday",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "maxUvIndex": {
        "value": 1,
        "description": "Day Maximum UV Index",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "daySignificantWeatherCode": {
        "value": "Overcast",
        "description": "Day Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "dayMaxScreenTemperature": {
        "value": 12.0,
        "description": "Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxTemp": {
        "value": 13.71,
        "description": "Upper Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxTemp": {
        "value": 10.23,
        "description": "Lower Bound on Day Maximum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayMaxFeelsLikeTemp": {
        "value": 10.6,
        "description": "Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayUpperBoundMaxFeelsLikeTemp": {
        "value": 11.49,
        "description": "Upper Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayLowerBoundMaxFeelsLikeTemp": {
        "value": 8.38,
        "description": "Lower Bound on Day Maximum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "dayProbabilityOfPrecipitation": {
        "value": 18,
        "description": "Probability of Precipitation During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfRain": {
        "value": 18,
        "description": "Probability of Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHeavyRain": {
        "value": 5,
        "description": "Probability of Heavy Rain During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "dayProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Day",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}

EXPECTED_FIRST_TWICE_DAILY_TIMESTEP_RAW_WEATHER_CODE = {
    "time": datetime.datetime(2024, 2, 16, 0, 0, tzinfo=datetime.timezone.utc),
    "midnight10MWindSpeed": {
        "value": 1.39,
        "description": "10m Wind Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnight10MWindDirection": {
        "value": 243,
        "description": "10m Wind Direction at Local Midnight",
        "unit_name": "degrees",
        "unit_symbol": "deg",
    },
    "midnight10MWindGust": {
        "value": 7.2,
        "description": "10m Wind Gust Speed at Local Midnight",
        "unit_name": "metres per second",
        "unit_symbol": "m/s",
    },
    "midnightVisibility": {
        "value": 27712,
        "description": "Visibility at Local Midnight",
        "unit_name": "metres",
        "unit_symbol": "m",
    },
    "midnightRelativeHumidity": {
        "value": 80.91,
        "description": "Relative Humidity at Local Midnight",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "midnightMslp": {
        "value": 102640,
        "description": "Mean Sea Level Pressure at Local Midnight",
        "unit_name": "pascals",
        "unit_symbol": "Pa",
    },
    "nightSignificantWeatherCode": {
        "value": 7,
        "description": "Night Significant Weather Code",
        "unit_name": "dimensionless",
        "unit_symbol": "1",
    },
    "nightMinScreenTemperature": {
        "value": 5.32,
        "description": "Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinTemp": {
        "value": 9.17,
        "description": "Upper Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinTemp": {
        "value": 3.56,
        "description": "Lower Bound on Night Minimum Screen Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightMinFeelsLikeTemp": {
        "value": 6.27,
        "description": "Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightUpperBoundMinFeelsLikeTemp": {
        "value": 8.74,
        "description": "Upper Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightLowerBoundMinFeelsLikeTemp": {
        "value": 2.75,
        "description": "Lower Bound on Night Minimum Feels Like Air Temperature",
        "unit_name": "degrees Celsius",
        "unit_symbol": "Cel",
    },
    "nightProbabilityOfPrecipitation": {
        "value": 11,
        "description": "Probability of Precipitation During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSnow": {
        "value": 0,
        "description": "Probability of Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavySnow": {
        "value": 0,
        "description": "Probability of Heavy Snow During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfRain": {
        "value": 10,
        "description": "Probability of Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHeavyRain": {
        "value": 0,
        "description": "Probability of Heavy Rain During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfHail": {
        "value": 0,
        "description": "Probability of Hail During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
    "nightProbabilityOfSferics": {
        "value": 0,
        "description": "Probability of Sferics During The Night",
        "unit_name": "percentage",
        "unit_symbol": "%",
    },
}



#!/usr/bin/env python
"""
This example checks whether it is due to rain at any point
today and then decides if we need to take an umbrella.
"""

import datetime

import datapoint

# Create umbrella variable to use later
umbrella = False

# Create datapoint connection
manager = datapoint.Manager(api_key="aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")

# Get a forecast for the nearest site
forecast = manager.get_forecast(51.500728, -0.124626, "hourly")

# Loop through all the timesteps in day 0 (today)
for timestep in forecast.timesteps:
    # Check to see if the chance of rain is more than 20% at any point
    if (
        timestep["probOfPrecipitation"]["value"] > 20
        and timestep["time"].date == datetime.date.now()
    ):
        umbrella = True

# Print out the results
if umbrella is True:
    print("Looks like rain! Better take an umbrella.")
else:
    print("Don't worry you don't need an umbrella today.")



#!/usr/bin/env python
"""
A variation on current_weather.py which uses postcodes rather than lon lat.
"""

import postcodes_io_api

import datapoint

# Create datapoint connection
manager = datapoint.Manager(api_key="aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")


# Get longitude and latitude from postcode
postcodes_conn = postcodes_io_api.Api()
postcode = postcodes_conn.get_postcode("SW1A 2AA")
latitude = postcode["result"]["latitude"]
longitude = postcode["result"]["longitude"]

# Get a forecast for the nearest site
forecast = manager.get_forecast(longitude, latitude, "hourly")

# Get the current timestep using now() and print out some info
now = forecast.now()
print(now["significantWeatherCode"])
print(f"{now['screenTemperature']['value']} {now['screenTemperature']['unit_symbol']}")



import datetime

import geojson
import pytest

import tests.reference_data.reference_data_test_forecast as reference_data_test_forecast
from datapoint import Forecast
from datapoint.exceptions import APIException

# TODO look into pytest-cases. Should reduce the amount of stored data structures


@pytest.fixture
def load_hourly_json():
    with open("./tests/reference_data/hourly_api_data.json") as f:
        my_json = geojson.load(f)
    return my_json


@pytest.fixture
def load_daily_json():
    with open("./tests/reference_data/daily_api_data.json") as f:
        my_json = geojson.load(f)
    return my_json


@pytest.fixture
def load_three_hourly_json():
    with open("./tests/reference_data/three_hourly_api_data.json") as f:
        my_json = geojson.load(f)
    return my_json


@pytest.fixture
def daily_forecast(load_daily_json):
    return Forecast.Forecast("daily", load_daily_json, convert_weather_code=True)


@pytest.fixture
def daily_forecast_raw_weather_code(load_daily_json):
    return Forecast.Forecast("daily", load_daily_json, convert_weather_code=False)


@pytest.fixture
def twice_daily_forecast(load_daily_json):
    return Forecast.Forecast("twice-daily", load_daily_json, convert_weather_code=True)


@pytest.fixture
def twice_daily_forecast_raw_weather_code(load_daily_json):
    return Forecast.Forecast("twice-daily", load_daily_json, convert_weather_code=False)


@pytest.fixture
def hourly_forecast(load_hourly_json):
    return Forecast.Forecast("hourly", load_hourly_json, convert_weather_code=True)


@pytest.fixture
def hourly_forecast_raw_weather_code(load_hourly_json):
    return Forecast.Forecast("hourly", load_hourly_json, convert_weather_code=False)


@pytest.fixture
def three_hourly_forecast(load_three_hourly_json):
    return Forecast.Forecast(
        "three-hourly", load_three_hourly_json, convert_weather_code=True
    )


@pytest.fixture
def hourly_first_forecast_and_parameters(load_hourly_json):
    parameters = load_hourly_json["parameters"][0]
    forecast = load_hourly_json["features"][0]["properties"]["timeSeries"][0]
    return (forecast, parameters)


@pytest.fixture
def three_hourly_first_forecast_and_parameters(load_three_hourly_json):
    parameters = load_three_hourly_json["parameters"][0]
    forecast = load_three_hourly_json["features"][0]["properties"]["timeSeries"][0]
    return (forecast, parameters)


@pytest.fixture
def expected_first_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_HOURLY_TIMESTEP


@pytest.fixture
def expected_first_hourly_timestep_raw_weather_code():
    return reference_data_test_forecast.EXPECTED_FIRST_HOURLY_TIMESTEP_RAW_WEATHER_CODE


@pytest.fixture
def expected_at_datetime_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_HOURLY_TIMESTEP


@pytest.fixture
def expected_at_datetime_hourly_final_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_HOURLY_FINAL_TIMESTEP


@pytest.fixture
def expected_first_daily_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_DAILY_TIMESTEP


@pytest.fixture
def expected_first_daily_timestep_raw_weather_code():
    return reference_data_test_forecast.EXPECTED_FIRST_DAILY_TIMESTEP_RAW_WEATHER_CODE


@pytest.fixture
def expected_at_datetime_daily_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_DAILY_TIMESTEP


@pytest.fixture
def expected_at_datetime_daily_final_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_DAILY_FINAL_TIMESTEP


@pytest.fixture
def expected_first_twice_daily_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_TWICE_DAILY_TIMESTEP


@pytest.fixture
def expected_first_twice_daily_timestep_raw_weather_code():
    return (
        reference_data_test_forecast.EXPECTED_FIRST_TWICE_DAILY_TIMESTEP_RAW_WEATHER_CODE
    )


@pytest.fixture
def expected_at_datetime_twice_daily_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_TWICE_DAILY_TIMESTEP


@pytest.fixture
def expected_at_datetime_twice_daily_final_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_TWICE_DAILY_FINAL_TIMESTEP


@pytest.fixture
def expected_first_three_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_THREE_HOURLY_TIMESTEP


@pytest.fixture
def expected_at_datetime_three_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_THREE_HOURLY_TIMESTEP


@pytest.fixture
def expected_at_datetime_three_hourly_final_timestep():
    return reference_data_test_forecast.EXPECTED_AT_DATETIME_THREE_HOURLY_FINAL_TIMESTEP


class TestHourlyForecast:
    def test_forecast_frequency(self, hourly_forecast):
        assert hourly_forecast.frequency == "hourly"

    def test_forecast_location_name(self, hourly_forecast):
        assert hourly_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, hourly_forecast):
        assert hourly_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, hourly_forecast):
        assert hourly_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, hourly_forecast):
        assert hourly_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, hourly_forecast):
        assert hourly_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, hourly_forecast, expected_first_hourly_timestep
    ):
        assert hourly_forecast.timesteps[0] == expected_first_hourly_timestep

    def test_build_timestep(
        self,
        hourly_forecast,
        hourly_first_forecast_and_parameters,
        expected_first_hourly_timestep,
    ):
        built_timestep = hourly_forecast._build_timestep(
            hourly_first_forecast_and_parameters[0],
            hourly_first_forecast_and_parameters[1],
        )

        assert built_timestep == expected_first_hourly_timestep

    def test_at_datetime(self, hourly_forecast, expected_at_datetime_hourly_timestep):
        ts = hourly_forecast.at_datetime(datetime.datetime(2024, 2, 16, 19, 15))
        assert ts == expected_at_datetime_hourly_timestep

    def test_at_datetime_final_timestamp(
        self, hourly_forecast, expected_at_datetime_hourly_final_timestep
    ):
        ts = hourly_forecast.at_datetime(datetime.datetime(2024, 2, 17, 19, 20))
        assert ts == expected_at_datetime_hourly_final_timestep

    def test_requested_time_too_early(self, hourly_forecast):
        with pytest.raises(APIException):
            hourly_forecast.at_datetime(datetime.datetime(2024, 2, 15, 18, 25))

    def test_requested_time_too_late(self, hourly_forecast):
        with pytest.raises(APIException):
            hourly_forecast.at_datetime(datetime.datetime(2024, 2, 17, 19, 35))

    def test_forecast_first_timestep_raw_weather_code(
        self,
        hourly_forecast_raw_weather_code,
        expected_first_hourly_timestep_raw_weather_code,
    ):
        assert (
            hourly_forecast_raw_weather_code.timesteps[0]
            == expected_first_hourly_timestep_raw_weather_code
        )


class TestDailyForecast:
    def test_forecast_frequency(self, daily_forecast):
        assert daily_forecast.frequency == "daily"

    def test_forecast_location_name(self, daily_forecast):
        assert daily_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, daily_forecast):
        assert daily_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, daily_forecast):
        assert daily_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, daily_forecast):
        assert daily_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, daily_forecast):
        assert daily_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, daily_forecast, expected_first_daily_timestep
    ):
        assert daily_forecast.timesteps[0] == expected_first_daily_timestep

    # Need a new test_build_timestep function here
    def test_build_timesteps(
        self, daily_forecast, load_daily_json, expected_first_daily_timestep
    ):
        timestep = daily_forecast._build_timestep(
            load_daily_json["features"][0]["properties"]["timeSeries"][0],
            load_daily_json["parameters"][0],
        )
        assert timestep == expected_first_daily_timestep

    def test_at_datetime(self, daily_forecast, expected_at_datetime_daily_timestep):
        ts = daily_forecast.at_datetime(datetime.datetime(2024, 2, 16, 19, 15))
        assert ts == expected_at_datetime_daily_timestep

    def test_at_datetime_final_timestamp(
        self, daily_forecast, expected_at_datetime_daily_final_timestep
    ):
        ts = daily_forecast.at_datetime(datetime.datetime(2024, 2, 17, 17))
        assert ts == expected_at_datetime_daily_final_timestep

    def test_requested_time_too_early(self, daily_forecast):
        with pytest.raises(APIException):
            daily_forecast.at_datetime(datetime.datetime(2024, 2, 15, 17))

    def test_requested_time_too_late(self, daily_forecast):
        with pytest.raises(APIException):
            daily_forecast.at_datetime(datetime.datetime(2024, 2, 23, 19))

    def test_forecast_first_timestep_raw_weather_code(
        self,
        daily_forecast_raw_weather_code,
        expected_first_daily_timestep_raw_weather_code,
    ):
        assert (
            daily_forecast_raw_weather_code.timesteps[0]
            == expected_first_daily_timestep_raw_weather_code
        )


class TestTwiceDailyForecast:
    def test_forecast_frequency(self, twice_daily_forecast):
        assert twice_daily_forecast.frequency == "twice-daily"

    def test_forecast_location_name(self, twice_daily_forecast):
        assert twice_daily_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, twice_daily_forecast):
        assert twice_daily_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, twice_daily_forecast):
        assert twice_daily_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, twice_daily_forecast):
        assert twice_daily_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, twice_daily_forecast):
        assert twice_daily_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, twice_daily_forecast, expected_first_twice_daily_timestep
    ):
        assert twice_daily_forecast.timesteps[0] == expected_first_twice_daily_timestep

    # twice-daily forecasts take daily data from DataHub
    def test_build_twice_daily_timesteps(
        self, twice_daily_forecast, load_daily_json, expected_first_twice_daily_timestep
    ):
        timesteps = twice_daily_forecast._build_twice_daily_timesteps(
            load_daily_json["features"][0]["properties"]["timeSeries"],
            load_daily_json["parameters"][0],
        )
        assert timesteps[0] == expected_first_twice_daily_timestep

    def test_at_datetime(
        self, twice_daily_forecast, expected_at_datetime_twice_daily_timestep
    ):
        ts = twice_daily_forecast.at_datetime(datetime.datetime(2024, 2, 16, 19, 15))
        assert ts == expected_at_datetime_twice_daily_timestep

    def test_at_datetime_final_timestep(
        self, twice_daily_forecast, expected_at_datetime_twice_daily_final_timestep
    ):
        ts = twice_daily_forecast.at_datetime(datetime.datetime(2024, 2, 17, 17))
        assert ts == expected_at_datetime_twice_daily_final_timestep

    def test_requested_time_too_early(self, twice_daily_forecast):
        with pytest.raises(APIException):
            twice_daily_forecast.at_datetime(datetime.datetime(2024, 2, 15, 17))

    def test_requested_time_too_late(self, twice_daily_forecast):
        with pytest.raises(APIException):
            twice_daily_forecast.at_datetime(datetime.datetime(2024, 2, 23, 19))

    def test_forecast_first_timestep_raw_weather_code(
        self,
        twice_daily_forecast_raw_weather_code,
        expected_first_twice_daily_timestep_raw_weather_code,
    ):
        print(twice_daily_forecast_raw_weather_code.timesteps[0])
        assert (
            twice_daily_forecast_raw_weather_code.timesteps[0]
            == expected_first_twice_daily_timestep_raw_weather_code
        )


class TestThreeHourlyForecast:
    def test_forecast_frequency(self, three_hourly_forecast):
        assert three_hourly_forecast.frequency == "three-hourly"

    def test_forecast_location_name(self, three_hourly_forecast):
        assert three_hourly_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, three_hourly_forecast):
        assert three_hourly_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, three_hourly_forecast):
        assert three_hourly_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, three_hourly_forecast):
        assert three_hourly_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, three_hourly_forecast):
        assert three_hourly_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, three_hourly_forecast, expected_first_three_hourly_timestep
    ):
        assert (
            three_hourly_forecast.timesteps[0] == expected_first_three_hourly_timestep
        )

    def test_build_timestep(
        self,
        three_hourly_forecast,
        three_hourly_first_forecast_and_parameters,
        expected_first_three_hourly_timestep,
    ):
        built_timestep = three_hourly_forecast._build_timestep(
            three_hourly_first_forecast_and_parameters[0],
            three_hourly_first_forecast_and_parameters[1],
        )

        assert built_timestep == expected_first_three_hourly_timestep

    def test_at_datetime(
        self, three_hourly_forecast, expected_at_datetime_three_hourly_timestep
    ):
        ts = three_hourly_forecast.at_datetime(datetime.datetime(2024, 2, 22, 19, 15))
        assert ts == expected_at_datetime_three_hourly_timestep

    def test_at_datetime_final_timestamp(
        self, three_hourly_forecast, expected_at_datetime_three_hourly_final_timestep
    ):
        ts = three_hourly_forecast.at_datetime(datetime.datetime(2024, 2, 24, 16))
        assert ts == expected_at_datetime_three_hourly_final_timestep

    def test_requested_time_too_early(self, three_hourly_forecast):
        with pytest.raises(APIException):
            three_hourly_forecast.at_datetime(datetime.datetime(2024, 2, 17, 13, 20))

    def test_requested_time_too_late(self, three_hourly_forecast):
        with pytest.raises(APIException):
            three_hourly_forecast.at_datetime(datetime.datetime(2024, 2, 24, 17))



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import importlib

project = "datapoint-python"
copyright = "2024, Emily Price, Jacob Tomlinson"
author = "Emily Price, Jacob Tomlinson"

release = importlib.metadata.version("datapoint")
version = importlib.metadata.version("datapoint")

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
]

templates_path = ["_templates"]
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "alabaster"
html_static_path = ["_static"]



#!/usr/bin/env python
"""
This example will tell us which day would be best to hang out
our washing to dry.

We will loop over the next 5 days and decide whether it is
ok to hang out the washing. Then for the good days we will rank
them and print out the best.
"""

import datapoint

# Set thresholds
MAX_WIND = 31  # in mph. We don't want the washing to blow away
MAX_PRECIPITATION = 20  # Max chance of rain we will accept

# Variables for later
best_time = None
best_score = 0  # For simplicity the score will be temperature + wind speed

# Create datapoint connection
manager = datapoint.Manager(api_key="aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")

# Get a forecast for the nearest site
forecast = manager.get_forecast(51.500728, -0.124626, "daily")

# Loop through days
for day in forecast.days:
    # Get the 'Day' timestep
    if day.timesteps[0].name == "Day":
        timestep = day.timesteps[0]

        # If precipitation, wind speed and gust are less than their threshold
        if (
            timestep.precipitation.value < MAX_PRECIPITATION
            and timestep.wind_speed.value < MAX_WIND
            and timestep.wind_gust.value < MAX_WIND
        ):
            # Calculate the score for this timestep
            timestep_score = timestep.wind_speed.value + timestep.temperature.value

            # If this timestep scores better than the current best replace it
            if timestep_score > best_score:
                best_score = timestep_score
                best_day = day.date

for timestep in forecast.timesteps:
    # If precipitation, wind speed and gust are less than their threshold
    if (
        timestep.precipitation.value < MAX_PRECIPITATION
        and timestep.wind_speed.value < MAX_WIND
        and timestep.wind_gust.value < MAX_WIND
    ):
        # Calculate the score for this timestep
        timestep_score = (
            timestep["windSpeed10m"]["value"] + timestep["screenTemperature"]["value"]
        )

        # If this timestep scores better than the current best replace it
        if timestep_score > best_score:
            best_score = timestep_score
            best_time = timestep["time"]


# If best_day is still None then there are no good days
if best_time is None:
    print("Better use the tumble dryer")

# Otherwise print out the day
else:
    print(f"{best_time} is the best day with a score of {best_score}")



import pytest
import requests

import tests.reference_data.reference_data_test_forecast as reference_data_test_forecast
from datapoint.Manager import Manager


class MockResponseHourly:
    def __init__(self):
        with open("./tests/reference_data/hourly_api_data.json") as f:
            my_json = f.read()

        self.text = my_json

    @staticmethod
    def raise_for_status():
        pass


@pytest.fixture
def _mock_response_hourly(monkeypatch):
    def mock_get(*args, **kwargs):
        return MockResponseHourly()

    monkeypatch.setattr(requests.Session, "get", mock_get)


@pytest.fixture
def hourly_forecast(_mock_response_hourly):
    m = Manager(api_key="aaaaaaaaaaaaaaaaaaaaaaaaa")
    f = m.get_forecast(50.9992, 0.0154, frequency="hourly", convert_weather_code=True)
    return f


@pytest.fixture
def expected_first_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_HOURLY_TIMESTEP


class MockResponseThreeHourly:
    def __init__(self):
        with open("./tests/reference_data/three_hourly_api_data.json") as f:
            my_json = f.read()

        self.text = my_json

    @staticmethod
    def raise_for_status():
        pass


@pytest.fixture
def _mock_response_three_hourly(monkeypatch):
    def mock_get(*args, **kwargs):
        return MockResponseThreeHourly()

    monkeypatch.setattr(requests.Session, "get", mock_get)


@pytest.fixture
def three_hourly_forecast(_mock_response_three_hourly):
    m = Manager(api_key="aaaaaaaaaaaaaaaaaaaaaaaaa")
    f = m.get_forecast(
        50.9992, 0.0154, frequency="three-hourly", convert_weather_code=True
    )
    return f


@pytest.fixture
def expected_first_three_hourly_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_THREE_HOURLY_TIMESTEP


class MockResponseDaily:
    def __init__(self):
        with open("./tests/reference_data/daily_api_data.json") as f:
            my_json = f.read()

        self.text = my_json

    @staticmethod
    def raise_for_status():
        pass


@pytest.fixture
def _mock_response_daily(monkeypatch):
    def mock_get(*args, **kwargs):
        return MockResponseDaily()

    monkeypatch.setattr(requests.Session, "get", mock_get)


@pytest.fixture
def daily_forecast(_mock_response_daily):
    m = Manager(api_key="aaaaaaaaaaaaaaaaaaaaaaaaa")
    f = m.get_forecast(50.9992, 0.0154, frequency="daily", convert_weather_code=True)
    return f


@pytest.fixture
def twice_daily_forecast(_mock_response_daily):
    m = Manager(api_key="aaaaaaaaaaaaaaaaaaaaaaaaa")
    f = m.get_forecast(
        50.9992, 0.0154, frequency="twice-daily", convert_weather_code=True
    )
    return f


@pytest.fixture
def expected_first_daily_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_DAILY_TIMESTEP


@pytest.fixture
def expected_first_twice_daily_timestep():
    return reference_data_test_forecast.EXPECTED_FIRST_TWICE_DAILY_TIMESTEP


class TestHourly:
    def test_location_name(self, hourly_forecast):
        assert hourly_forecast.name == "Sheffield Park"

    def test_forecast_frequency(self, hourly_forecast):
        assert hourly_forecast.frequency == "hourly"

    def test_forecast_location_latitude(self, hourly_forecast):
        assert hourly_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, hourly_forecast):
        assert hourly_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, hourly_forecast):
        assert hourly_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, hourly_forecast):
        assert hourly_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, hourly_forecast, expected_first_hourly_timestep
    ):
        assert hourly_forecast.timesteps[0] == expected_first_hourly_timestep


class TestThreeHourly:
    def test_forecast_frequency(self, three_hourly_forecast):
        assert three_hourly_forecast.frequency == "three-hourly"

    def test_forecast_location_name(self, three_hourly_forecast):
        assert three_hourly_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, three_hourly_forecast):
        assert three_hourly_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, three_hourly_forecast):
        assert three_hourly_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, three_hourly_forecast):
        assert three_hourly_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, three_hourly_forecast):
        assert three_hourly_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, three_hourly_forecast, expected_first_three_hourly_timestep
    ):
        assert (
            three_hourly_forecast.timesteps[0] == expected_first_three_hourly_timestep
        )


class TestDaily:
    def test_forecast_frequency(self, daily_forecast):
        assert daily_forecast.frequency == "daily"

    def test_forecast_location_name(self, daily_forecast):
        assert daily_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, daily_forecast):
        assert daily_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, daily_forecast):
        assert daily_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, daily_forecast):
        assert daily_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, daily_forecast):
        assert daily_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, daily_forecast, expected_first_daily_timestep
    ):
        assert daily_forecast.timesteps[0] == expected_first_daily_timestep


class TestTwiceDaily:
    def test_forecast_frequency(self, twice_daily_forecast):
        assert twice_daily_forecast.frequency == "twice-daily"

    def test_forecast_location_name(self, twice_daily_forecast):
        assert twice_daily_forecast.name == "Sheffield Park"

    def test_forecast_location_latitude(self, twice_daily_forecast):
        assert twice_daily_forecast.forecast_latitude == 50.9992

    def test_forecast_location_longitude(self, twice_daily_forecast):
        assert twice_daily_forecast.forecast_longitude == 0.0154

    def test_forecast_distance_from_request(self, twice_daily_forecast):
        assert twice_daily_forecast.distance_from_requested_location == 1081.5349

    def test_forecast_elevation(self, twice_daily_forecast):
        assert twice_daily_forecast.elevation == 37.0

    def test_forecast_first_timestep(
        self, twice_daily_forecast, expected_first_twice_daily_timestep
    ):
        assert twice_daily_forecast.timesteps[0] == expected_first_twice_daily_timestep



import datetime

from datapoint.exceptions import APIException
from datapoint.weather_codes import WEATHER_CODES


class Forecast:
    """Forecast data returned from DataHub

    Provides access to forecasts as far ahead as provided by DataHub. See the
    DataHub documentation for the latest limits on the forecast range. The
    values of data from DataHub are combined with the unit information and
    description and returned as a dict.

    Basic Usage::

      >>> import datapoint
      >>> m = datapoint.Manager.Manager(api_key = "blah")
      >>> f = m.get_forecast(
                  latitude=50,
                  longitude=0,
                  frequency="hourly",
                  convert_weather_code=True,
              )
      >>> f.now()
      {
          'time': datetime.datetime(2024, 2, 19, 13, 0, tzinfo=datetime.timezone.utc),
          'screenTemperature': {
              'value': 10.09,
              'description': 'Screen Air Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'screenDewPointTemperature': {
              'value': 8.08,
              'description': 'Screen Dew Point Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'feelsLikeTemperature': {
              'value': 6.85,
              'description': 'Feels Like Temperature',
              'unit_name': 'degrees Celsius',
              'unit_symbol': 'Cel'
          },
          'windSpeed10m': {
              'value': 7.57,
              'description': '10m Wind Speed',
              'unit_name': 'metres per second',
              'unit_symbol': 'm/s'
          },
          'windDirectionFrom10m': {
              'value': 263,
              'description': '10m Wind From Direction',
              'unit_name': 'degrees',
              'unit_symbol': 'deg'
          },
          'windGustSpeed10m': {
              'value': 12.31,
              'description': '10m Wind Gust Speed',
              'unit_name': 'metres per second',
              'unit_symbol': 'm/s'
          },
          'visibility': {
              'value': 21201,
              'description': 'Visibility',
              'unit_name': 'metres',
              'unit_symbol': 'm'
          },
          'screenRelativeHumidity': {
              'value': 87.81,
              'description': 'Screen Relative Humidity',
              'unit_name': 'percentage',
              'unit_symbol': '%'
          },
          'mslp': {
              'value': 103080,
              'description': 'Mean Sea Level Pressure',
              'unit_name': 'pascals',
              'unit_symbol': 'Pa'
          },
          'uvIndex': {
              'value': 1,
              'description': 'UV Index',
              'unit_name': 'dimensionless',
              'unit_symbol': '1'
          },
          'significantWeatherCode': {
              'value': 'Cloudy',
              'description': 'Significant Weather Code',
              'unit_name': 'dimensionless',
              'unit_symbol': '1'
          },
          'precipitationRate': {
              'value': 0.0,
              'description': 'Precipitation Rate',
              'unit_name': 'millimetres per hour',
              'unit_symbol': 'mm/h'
          },
          'probOfPrecipitation': {
              'value': 21,
              'description': 'Probability of Precipitation',
              'unit_name': 'percentage',
              'unit_symbol': '%'
          }
      }
    """

    def __init__(self, frequency, api_data, convert_weather_code):
        """
        :param frequency: Frequency of forecast: 'hourly', 'three-hourly',
            'twice-daily', 'daily'
        :param api_data: Data returned from API call
        :param: convert_weather_code: Convert numeric weather codes to string description
        :type frequency: string
        :type api_data: dict
        :type convert_weather_code: bool
        """
        self.frequency = frequency
        # Need to parse format like 2024-02-17T15:00Z. This can only be
        # done with datetime.datetime.fromisoformat from python 3.11
        # onwards.
        self.data_date = datetime.datetime.strptime(
            api_data["features"][0]["properties"]["modelRunDate"],
            "%Y-%m-%dT%H:%M%z",
        )  #: The date the provided forecast was generated.

        self.forecast_longitude = api_data["features"][0]["geometry"]["coordinates"][
            0
        ]  #: The longitude of the provided forecast.
        self.forecast_latitude = api_data["features"][0]["geometry"]["coordinates"][
            1
        ]  #: The latitude of the provided forecast.
        self.distance_from_requested_location = api_data["features"][0]["properties"][
            "requestPointDistance"
        ]  #: The distance of the location of the provided forecast from the requested location
        self.name = api_data["features"][0]["properties"]["location"][
            "name"
        ]  #: The name of the location of the provided forecast

        # N.B. Elevation is in metres above or below the WGS 84 reference
        # ellipsoid as per GeoJSON spec.
        self.elevation = api_data["features"][0]["geometry"]["coordinates"][
            2
        ]  #: The elevation of the location of the provided forecast

        self.convert_weather_code = (
            convert_weather_code  #: Convert numeric weather codes to string description
        )

        forecasts = api_data["features"][0]["properties"]["timeSeries"]
        parameters = api_data["parameters"][0]
        if frequency == "twice-daily":
            self.timesteps = self._build_twice_daily_timesteps(forecasts, parameters)
        else:
            self.timesteps = []
            for forecast in forecasts:
                self.timesteps.append(self._build_timestep(forecast, parameters))

    def _build_twice_daily_timesteps(self, forecasts, parameters):
        """Build individual timesteps from forecasts and metadata

        Take the forecast data from DataHub and combine with unit information
        in each timestep. Break each day into day and night steps. ASSUME that
        each step has data for the night referred to in the timestamp and the
        following dawn-dusk period.

        :parameter forecasts: Forecast data from DataHub
        :parameter parameters: Unit information from DataHub
        :type forecasts: list
        :type parameters: dict

        :return: List of timesteps
        :rtype: list
        """

        timesteps = []
        for forecast in forecasts:
            # Need to parse format like 2024-02-17T15:00Z. This can only be
            # done with datetime.datetime.fromisoformat from python 3.11
            # onwards.
            night_step = {
                "time": datetime.datetime.strptime(forecast["time"], "%Y-%m-%dT%H:%M%z")
            }
            day_step = {
                "time": datetime.datetime.strptime(forecast["time"], "%Y-%m-%dT%H:%M%z")
                + datetime.timedelta(hours=12)
            }

            for element, value in forecast.items():
                if element.startswith("midday"):
                    day_step[element] = {
                        "value": value,
                        "description": parameters[element]["description"],
                        "unit_name": parameters[element]["unit"]["label"],
                        "unit_symbol": parameters[element]["unit"]["symbol"]["type"],
                    }
                elif element.startswith("midnight"):
                    night_step[element] = {
                        "value": value,
                        "description": parameters[element]["description"],
                        "unit_name": parameters[element]["unit"]["label"],
                        "unit_symbol": parameters[element]["unit"]["symbol"]["type"],
                    }
                elif element.startswith("day"):
                    if (
                        element == "daySignificantWeatherCode"
                        and self.convert_weather_code
                    ):
                        day_step[element] = {
                            "value": WEATHER_CODES[str(value)],
                            "description": parameters[element]["description"],
                            "unit_name": parameters[element]["unit"]["label"],
                            "unit_symbol": parameters[element]["unit"]["symbol"][
                                "type"
                            ],
                        }

                    else:
                        day_step[element] = {
                            "value": value,
                            "description": parameters[element]["description"],
                            "unit_name": parameters[element]["unit"]["label"],
                            "unit_symbol": parameters[element]["unit"]["symbol"][
                                "type"
                            ],
                        }
                elif element.startswith("night"):
                    if (
                        element == "nightSignificantWeatherCode"
                        and self.convert_weather_code
                    ):
                        night_step[element] = {
                            "value": WEATHER_CODES[str(value)],
                            "description": parameters[element]["description"],
                            "unit_name": parameters[element]["unit"]["label"],
                            "unit_symbol": parameters[element]["unit"]["symbol"][
                                "type"
                            ],
                        }

                    else:
                        night_step[element] = {
                            "value": value,
                            "description": parameters[element]["description"],
                            "unit_name": parameters[element]["unit"]["label"],
                            "unit_symbol": parameters[element]["unit"]["symbol"][
                                "type"
                            ],
                        }
                elif element == "maxUvIndex":
                    day_step[element] = {
                        "value": value,
                        "description": parameters[element]["description"],
                        "unit_name": parameters[element]["unit"]["label"],
                        "unit_symbol": parameters[element]["unit"]["symbol"]["type"],
                    }

            timesteps.append(night_step)
            timesteps.append(day_step)

        timesteps = sorted(timesteps, key=lambda t: t["time"])
        return timesteps

    def _build_timestep(self, forecast, parameters):
        """Build individual timestep from forecast and metadata

        Take the forecast data from DataHub for a single time and combine with
        unit information in each timestep.

        :parameter forecast: Forecast data from DataHub
        :parameter parameters: Unit information from DataHub
        :type forecast: dict
        :type parameters:dict

        :return: Individual forecast timestep
        :rtype: dict

        """

        timestep = {}
        for element, value in forecast.items():
            if element == "time":
                # Need to parse format like 2024-02-17T15:00Z. This can only be
                # done with datetime.datetime.fromisoformat from python 3.11
                # onwards.
                timestep["time"] = datetime.datetime.strptime(
                    forecast["time"], "%Y-%m-%dT%H:%M%z"
                )

            elif (
                element
                in (
                    "significantWeatherCode",
                    "daySignificantWeatherCode",
                    "nightSignificantWeatherCode",
                )
            ) and self.convert_weather_code:
                timestep[element] = {
                    "value": WEATHER_CODES[str(value)],
                    "description": parameters[element]["description"],
                    "unit_name": parameters[element]["unit"]["label"],
                    "unit_symbol": parameters[element]["unit"]["symbol"]["type"],
                }
            else:
                timestep[element] = {
                    "value": value,
                    "description": parameters[element]["description"],
                    "unit_name": parameters[element]["unit"]["label"],
                    "unit_symbol": parameters[element]["unit"]["symbol"]["type"],
                }

        return timestep

    def _check_requested_time(self, target):
        """Check that a forecast for the requested time can be provided

        :parameter target: The requested time for the forecast
        :type target: datetime
        """
        # Check that there is a forecast for the requested time.
        # If we have an hourly forecast, check that the requested time is at
        # most 30 minutes before the first datetime we have a forecast for.
        if self.frequency == "hourly" and target < self.timesteps[0][
            "time"
        ] - datetime.timedelta(hours=0, minutes=30):
            err_str = (
                "There is no forecast available for the requested time. "
                "The requested time is more than 30 minutes before the "
                "first available forecast."
            )
            raise APIException(err_str)

        # If we have a three-hourly forecast, check that the requested time is at
        # most 1.5 hours before the first datetime we have a forecast for.
        if self.frequency == "three-hourly" and target < self.timesteps[0][
            "time"
        ] - datetime.timedelta(hours=1, minutes=30):
            err_str = (
                "There is no forecast available for the requested time. "
                "The requested time is more than 1 hour and 30 minutes "
                "before the first available forecast."
            )
            raise APIException(err_str)

        # If we have a daily forecast, check that the requested time is at
        # most 6 hours before the first datetime we have a forecast for.
        if self.frequency == "daily" and target < self.timesteps[0][
            "time"
        ] - datetime.timedelta(hours=6):
            err_str = (
                "There is no forecast available for the requested time. "
                "The requested time is more than 6 hours before the first "
                "available forecast."
            )

            raise APIException(err_str)

        # If we have a twice-daily forecast, check that the requested time is
        # at most 6 hours before the first datetime we have a forecast for.
        if self.frequency == "twice-daily" and target < self.timesteps[0][
            "time"
        ] - datetime.timedelta(hours=6):
            err_str = (
                "There is no forecast available for the requested time. "
                "The requested time is more than 6 hours before the first "
                "available forecast."
            )

            raise APIException(err_str)

        # If we have an hourly forecast, check that the requested time is at
        # most 30 minutes after the final datetime we have a forecast for
        if self.frequency == "hourly" and target > (
            self.timesteps[-1]["time"] + datetime.timedelta(hours=0, minutes=30)
        ):
            err_str = (
                "There is no forecast available for the requested time. The "
                "requested time is more than 30 minutes after the first "
                "available forecast"
            )

            raise APIException(err_str)

        # If we have a three-hourly forecast, then the target must be within 1.5
        # hours of the last timestep
        if self.frequency == "three-hourly" and target > (
            self.timesteps[-1]["time"] + datetime.timedelta(hours=1, minutes=30)
        ):
            err_str = (
                "There is no forecast available for the requested time. The "
                "requested time is more than 1.5 hours after the first "
                "available forecast."
            )

            raise APIException(err_str)

        # If we have a daily forecast, then the target must be within 6 hours
        # of the last timestep
        if self.frequency == "daily" and target > (
            self.timesteps[-1]["time"] + datetime.timedelta(hours=6)
        ):
            err_str = (
                "There is no forecast available for the requested time. The "
                "requested time is more than 6 hours after the first available "
                "forecast."
            )

            raise APIException(err_str)

        # If we have a twice-daily forecast, then the target must be within 6 hours
        # of the last timestep
        if self.frequency == "twice-daily" and target > (
            self.timesteps[-1]["time"] + datetime.timedelta(hours=6)
        ):
            err_str = (
                "There is no forecast available for the requested time. The "
                "requested time is more than 6 hours after the first available "
                "forecast."
            )

            raise APIException(err_str)

    def at_datetime(self, target):
        """Return the timestep closest to the target datetime

        :parameter target: Time to get the forecast for
        :type target: datetime

        :return: Individual forecast timestep
        :rtype: dict

        """

        # Convert target to offset aware datetime
        if target.tzinfo is None:
            target = datetime.datetime.combine(
                target.date(), target.time(), self.timesteps[0]["time"].tzinfo
            )

        self._check_requested_time(target)

        # Loop over all timesteps
        # Calculate the first time difference
        prev_td = target - self.timesteps[0]["time"]
        prev_ts = self.timesteps[0]
        to_return = None

        for i, timestep in enumerate(self.timesteps, start=1):
            # Calculate the difference between the target time and the
            # timestep.
            td = target - timestep["time"]

            # Find the timestep which is further from the target than the
            # previous one. Return the previous timestep
            if abs(td.total_seconds()) > abs(prev_td.total_seconds()):
                # We are further from the target
                to_return = prev_ts
                break
            if i == len(self.timesteps):
                to_return = timestep

            prev_ts = timestep
            prev_td = td
        return to_return

    def now(self):
        """Return the closest timestep to the current time

        :return: Individual forecast timestep
        :rtype: dict
        """

        d = datetime.datetime.now(tz=self.timesteps[0]["time"].tzinfo)
        return self.at_datetime(d)

    def future(self, days=0, hours=0, minutes=0):
        """Return the closest timestep to a date in a given amount of time

        Either provide components of the time to the forecast or the total
        hours or minutes

        Providing components::

        >>> import datapoint
        >>> m = datapoint.Manager(api_key = "blah")
        >>> f = m.get_forecast(latitude=50, longitude=0, frequency="hourly")
        >>> f.future(days=1, hours=2)

        Providing total hours::

        >>> import datapoint
        >>> m = datapoint.Manager(api_key = "blah")
        >>> f = m.get_forecast(latitude=50, longitude=0, frequency="hourly")
        >>> f.future(hours=26)


        :parameter days: How many days ahead
        :parameter hours: How many hours ahead
        :parameter minutes: How many minutes ahead
        :type days: int
        :type hours: int
        :type minutes: int

        :return: Individual forecast timestep
        :rtype: dict
        """

        d = datetime.datetime.now(tz=self.timesteps[0]["time"].tzinfo)
        target = d + datetime.timedelta(days=days, hours=hours, minutes=minutes)

        return self.at_datetime(target)



#!/usr/bin/env python
"""
This one's for Londoners. Get the weather for home and work and get the tube status
for your usual line. Then use that information to decide whether you're better off
cycling or catching the tube.
"""

import tubestatus

import datapoint

# Create datapoint connection
manager = datapoint.Manager(api_key="aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")

# Get a forecast for my house and work
my_house_forecast = manager.get_forecast(51.5016730, 0.0057500, "hourly")
work_forecast = manager.get_forecast(51.5031650, -0.1123050, "hourly")

# Get the current timestep for both locations
my_house_now = my_house_forecast.now()
work_now = work_forecast.now()

# Create a tube status connection
current_status = tubestatus.Status()

# Get the status of the Waterloo and City line
waterloo_status = current_status.get_status("Waterloo and City")

# Check whether there are any problems with rain or the tube
if (
    my_house_now["probOfPrecipitation"]["value"] < 40
    and work_now["probOfPrecipitation"]["value"] < 40
    and waterloo_status.description == "Good Service"
):
    print("Rain is unlikely and tube service is good, the decision is yours.")

# If it is going to rain then suggest the tube
elif (
    my_house_now["probOfPrecipitation"]["value"] >= 40
    or work_now["probOfPrecipitation"]["value"] >= 40
) and waterloo_status.description == "Good Service":
    print("Looks like rain, better get the tube")

# If the tube isn't running then suggest cycling
elif (
    my_house_now["probOfPrecipitation"]["value"] < 40
    and work_now["probOfPrecipitation"]["value"] < 40
    and waterloo_status.description != "Good Service"
):
    print("Bad service on the tube, cycling it is!")

# Else if both are bad then suggest cycling in the rain
else:
    print(
        "The tube has poor service so you'll have to cycle,"
        " but it's raining so take your waterproofs."
    )



#!/usr/bin/env python
"""
This is a simple example which will print out the current weather and
temperature for our location.
"""

import datapoint

# Create datapoint connection
manager = datapoint.Manager(api_key="aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee")

# Get a forecast for the nearest location
forecast = manager.get_forecast(51.500728, -0.124626, "hourly")

# Get the current timestep using now() and print out some info
now = forecast.now()
print(now["significantWeatherCode"])
print(f"{now['screenTemperature']['value']} {now['screenTemperature']['unit_symbol']}")
